@@ -1,14 +1,12 @@
 from __future__ import annotations
 
-import base64
 import enum
 import json
-import logging
 import time
-from concurrent.futures import Future, ThreadPoolExecutor
+from concurrent.futures import Future
 from datetime import datetime
 from itertools import chain
-from typing import Any, Iterable, Iterator, List, Mapping, Optional, Sequence, Type, TypeVar, Union
+from typing import Any, Iterable, List, Mapping, Optional, Sequence, Type, TypeVar, Union
 from urllib.parse import urljoin
 
 import polars as pl
@@ -27,12 +25,14 @@ from chalk.client.client_protocol import (
     ResolverRunResponse,
     WhoAmIResponse,
 )
+from chalk.client.dataset import DatasetVersion, load_dataset
 from chalk.config.auth_config import load_token
-from chalk.features import DataFrame, Feature, Features, FeatureWrapper, unwrap_feature
+from chalk.features import DataFrame, Feature, Features, FeatureWrapper, ensure_feature, unwrap_feature
 from chalk.features.pseudofeatures import CHALK_TS_FEATURE
 from chalk.serialization.codec import FEATURE_CODEC
+from chalk.utils.log_with_context import get_logger
 
-_logger = logging.getLogger(__name__)
+_logger = get_logger(__name__)
 
 import pandas as pd
 
@@ -61,13 +61,6 @@ class _OfflineQueryInput(BaseModel):
     values: List[List[Any]]
 
 
-class _OfflineQueryRequest(BaseModel):
-    output: List[str]
-    input: Optional[_OfflineQueryInput] = None
-    dataset: Optional[str] = None
-    max_samples: Optional[int] = None
-
-
 class _QueryRequest(BaseModel):
     inputs: Mapping[str, Any]
     outputs: List[str]
@@ -83,10 +76,6 @@ class _TriggerResolverRunRequest(BaseModel):
     resolver_fqn: str
 
 
-class _GetRunStatusRequest(BaseModel):
-    run_id: str
-
-
 T = TypeVar("T")
 
 
@@ -102,10 +91,6 @@ class _OnlineQueryResponse(BaseModel):
     errors: Optional[List[ChalkError]]
 
 
-class _DatasetErrorResponse(BaseModel):
-    fatal_errors: List[ChalkError]
-
-
 class _CreateOfflineQueryJobRequest(BaseModel):
     """
     Attributes
@@ -145,6 +130,7 @@ class _GetOfflineQueryJobResponse(BaseModel):
     """
 
     is_finished: bool
+    version: int
     urls: List[str]
 
 
@@ -203,7 +189,7 @@ class OnlineQueryResponseWrapper(OnlineQueryResponse):
 
     def get_feature(self, feature: Any) -> Optional[FeatureResult]:
         # Typing `feature` as Any, as the Features will be typed as the underlying datatypes, not as Feature
-        return self._values.get(_get_feature_root_fqn(feature))
+        return self._values.get(ensure_feature(feature).root_fqn)
 
     def get_feature_value(self, feature: Any) -> Optional[Any]:
         # Typing `feature` as Any, as the Features will be typed as the underlying datatypes, not as Feature
@@ -211,34 +197,28 @@ class OnlineQueryResponseWrapper(OnlineQueryResponse):
         return v and v.value
 
 
-def _get_feature_root_fqn(feat: Union[str, Feature, FeatureWrapper, Any]) -> str:
-    if isinstance(feat, str):
-        return feat
-    elif isinstance(feat, FeatureWrapper):
-        return unwrap_feature(feat).root_fqn
-    elif isinstance(feat, Feature):
-        return feat.root_fqn
-    raise ValueError(f"Reference must be to a leaf feature. Received '{feat}'")
-
-
-def _expand_scalar_features_shallow(fs: Sequence[Any]) -> Iterator[str]:
-    for o in fs:
-        if isinstance(o, str):
+def _expand_scalar_features_shallow(o: Union[Feature, FeatureWrapper, Features, str, Any]) -> Iterable[Feature]:
+    if isinstance(o, str):
+        yield Feature.from_root_fqn(o)
+        return
+    if isinstance(o, type) and issubclass(o, Features):
+        for f in o.features:
+            if f.is_scalar:
+                yield f
+        return
+    if isinstance(o, FeatureWrapper):
+        o = unwrap_feature(o)
+    if isinstance(o, Feature):
+        if o.is_scalar:
             yield o
-        if isinstance(o, type) and issubclass(o, Features):
-            for f in o.features:
-                if f.is_scalar:
-                    yield f.root_fqn
-        if isinstance(o, FeatureWrapper):
-            o = unwrap_feature(o)
-
-        if isinstance(o, Feature) and o.is_scalar:
-            yield o.root_fqn
-        elif isinstance(o, Feature) and o.is_has_one:
+            return
+        if o.is_has_one:
             assert o.joined_class is not None
             for f in o.joined_class.features:
                 if isinstance(f, Feature) and f.is_scalar:
-                    yield f"{o.root_fqn}.{f.name}"
+                    yield Feature.from_root_fqn(f"{o.root_fqn}.{f.name}")
+            return
+    raise ValueError(f"Unsupported feature {o}")
 
 
 class ChalkAPIClientImpl(ChalkAPIClientProtocol):
@@ -298,11 +278,12 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
             ).dict(),
             timeout=10,
         )
+        resp.raise_for_status()
         response_json = resp.json()
         try:
             creds = _ExchangeCredentialsResponse(**response_json)
         except ValidationError:
-            raise HTTPError(response_json)
+            raise HTTPError(response=resp)
         self._default_headers["Authorization"] = f"Bearer {creds.access_token}"
         self._engines = creds.engines
         self._exchanged_credentials = True
@@ -373,8 +354,8 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
     def query(
         self,
         input: Mapping[Union[str, Feature, FeatureWrapper, Any], Any],
-        output: List[Union[str, Feature, Any]],
-        staleness: Optional[Mapping[Union[str, Feature, Any], str]] = None,
+        output: Sequence[Union[str, Feature, FeatureWrapper, Any]],
+        staleness: Optional[Mapping[Union[str, Feature, FeatureWrapper, Any], str]] = None,
         context: Optional[OnlineQueryContext] = None,
         preview_deployment_id: Optional[str] = None,
         correlation_id: Optional[str] = None,
@@ -383,13 +364,15 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
     ) -> OnlineQueryResponse:
         encoded_inputs = {}
         for feature, value in input.items():
-            fqn = _get_feature_root_fqn(feature)
-            encoded_inputs[fqn] = FEATURE_CODEC.encode_fqn(fqn, value)
-
+            feature = ensure_feature(feature)
+            encoded_inputs[feature.fqn] = FEATURE_CODEC.encode(feature, value)
+        outputs: List[str] = []
+        for x in output:
+            outputs.extend(f.root_fqn for f in _expand_scalar_features_shallow(x))
         request = _QueryRequest(
             inputs=encoded_inputs,
-            outputs=list(_expand_scalar_features_shallow(output)),
-            staleness=staleness and {_get_feature_root_fqn(k): v for k, v in staleness.items()},
+            outputs=outputs,
+            staleness={} if staleness is None else {ensure_feature(k).root_fqn: v for k, v in staleness.items()},
             context=context,
             deployment_id=preview_deployment_id,
             correlation_id=correlation_id,
@@ -464,7 +447,7 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
         columns = input.columns
         matrix = input.T.values.tolist()
 
-        columns_fqn = [_get_feature_root_fqn(c) for c in chain(columns, (CHALK_TS_FEATURE,))]
+        columns_fqn = [ensure_feature(c).root_fqn for c in chain(columns, (CHALK_TS_FEATURE,))]
 
         matrix.append([a for a in input_times])
 
@@ -480,7 +463,9 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
             values=matrix,
         )
 
-        query_output = list(_expand_scalar_features_shallow(output))
+        query_output: List[Feature] = []
+        for x in output:
+            query_output.extend(_expand_scalar_features_shallow(x))
 
         try:
             response = self._create_and_await_offline_query_job(
@@ -497,11 +482,13 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
 
     def sample(
         self,
-        output: List[Union[str, Feature, Any]],
+        output: List[Union[str, Feature, FeatureWrapper, Any]],
         max_samples: Optional[int] = None,
         context: Optional[OfflineQueryContext] = None,
     ) -> pd.DataFrame:
-        query_output = _expand_scalar_features_shallow(output)
+        query_output: list[Feature] = []
+        for x in output:
+            query_output.extend(_expand_scalar_features_shallow(x))
 
         try:
             response = self._create_and_await_offline_query_job(output=query_output, max_samples=max_samples)
@@ -555,30 +542,9 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
 
         return response
 
-    def _decode_col_name(self, col_name: str) -> str:
-        x_split = col_name.split("_")
-        if x_split[0] == "ca":
-            assert len(x_split) == 2
-            return x_split[1]
-        elif x_split[0] == "cb":
-            root_fqn_b32 = x_split[1]
-            return base64.b32decode(root_fqn_b32.replace("0", "=").upper()).decode("utf8") + "_".join(x_split[2:])
-        else:
-            raise ValueError(f"Unexpected identifier: {x_split[0]}")
-
-    def _decode_column_names(self, column_names: List[str], ts_fqn: Optional[str]) -> Mapping[str, str]:
-        ans: Mapping[str, str] = {}
-        for x in column_names:
-            if x.endswith("__"):
-                if x == "__observed_at__" and ts_fqn is not None:
-                    ans[x] = ts_fqn
-                continue
-            ans[x] = self._decode_col_name("_".join(x.split("_")[:2]))
-        return ans
-
     def _create_and_await_offline_query_job(
         self,
-        output: Iterable[str],
+        output: Sequence[Feature],
         input: Optional[_OfflineQueryInput] = None,
         max_samples: Optional[int] = None,
         dataset_name: Optional[str] = None,
@@ -587,7 +553,7 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
     ) -> pl.DataFrame:
         output = list(output)
         req = _CreateOfflineQueryJobRequest(
-            output=output,
+            output=[x.root_fqn for x in output],
             destination_format="PARQUET",
             input=input,
             max_samples=max_samples,
@@ -605,8 +571,11 @@ class ChalkAPIClientImpl(ChalkAPIClientProtocol):
             if status.is_finished:
                 break
             time.sleep(0.5)
-
-        executor = ThreadPoolExecutor(max_workers=16)
+        return load_dataset(
+            uris=status.urls,
+            version=DatasetVersion.BIGQUERY_JOB_WITH_B32_ENCODED_COLNAMES,
+            features=output,
+        )
 
         df_futures: list[Future[pl.DataFrame]] = []
         for f in status.urls:
