@@ -1,11 +1,13 @@
-from typing import Annotated, TypeVar
+from typing import TypeVar
 
 T = TypeVar("T")
 
+from typing_extensions import Annotated
+
 
 class PrimaryMeta(type):
     def __getitem__(self, item: T) -> Annotated[T, "primary"]:
-        return Annotated[T, "__chalk_primary__"]
+        return Annotated[item, "__chalk_primary__"]
 
 
 Primary = PrimaryMeta("Primary", (object,), {})
