@@ -19,8 +19,6 @@ from typing import (
     TypeVar,
     Union,
     cast,
-    get_args,
-    get_origin,
     get_type_hints,
     overload,
 )
@@ -28,7 +26,7 @@ from typing import (
 import polars
 import pyarrow
 from pydantic import BaseModel
-from typing_extensions import ParamSpec
+from typing_extensions import ParamSpec, get_args, get_origin
 
 from chalk.features.dataframe import DataFrame
 from chalk.features.feature_field import Feature
