@@ -4,7 +4,7 @@ import collections.abc
 import dataclasses
 import enum
 from datetime import date, datetime, timezone
-from typing import TYPE_CHECKING, Any, List, Type, Union, cast, get_origin, get_type_hints
+from typing import TYPE_CHECKING, Any, List, Type, Union, cast, get_type_hints
 
 import cattrs
 import pendulum
@@ -14,7 +14,7 @@ import pytz
 from dateutil import parser
 from pendulum.tz.timezone import Timezone
 from polars.internals.construction import is_namedtuple
-from typing_extensions import get_args
+from typing_extensions import get_args, get_origin
 
 from chalk.serialization.parsed_annotation import ParsedAnnotation
 from chalk.streams import Windowed
