@@ -74,6 +74,7 @@ class BaseSQLSource(BaseSQLSourceProtocol):
         self._session_maker = session_maker or DBSessionMaker()
         self._incremental_settings = None
         self.registry.append(self)
+        self._session = None
 
     def set_session_maker(self, maker: DBSessionMakerProtocol) -> None:
         self._session_maker = maker
@@ -112,12 +113,12 @@ class BaseSQLSource(BaseSQLSourceProtocol):
         args: Optional[Mapping[str, str]] = None,
     ) -> StringChalkQueryProtocol:
         fields = fields or {}
-        session = self._session_maker.get_session(self)
+        session = self._session or self._session_maker.get_session(self)
         fields = {f: unwrap_feature(v) if isinstance(v, FeatureWrapper) else v for (f, v) in fields.items()}
         return StringChalkQuery(session=session, source=self, query=query, fields=fields, args=args)
 
     def raw_session(self) -> Session:
-        return self._session_maker.get_session(self)._raw_session
+        return self._session._raw_session if self._session else self._session_maker.get_session(self)._raw_session
 
     def query(self, *entities, **kwargs) -> ChalkQueryProtocol:
         targets = []
@@ -139,7 +140,7 @@ class BaseSQLSource(BaseSQLSourceProtocol):
                         targets.append(feature_value.label(f.fqn))
             else:
                 targets.append(e)
-        session = self._session_maker.get_session(self)
+        session = self._session or self._session_maker.get_session(self)
         session.update_query(lambda x: x.query(*targets, **kwargs))
 
         return ChalkQuery(
