@@ -1,4 +1,4 @@
-from typing import Any, Dict, Optional, Union
+from typing import Any, Dict, Mapping, Optional, Union
 
 from sqlalchemy.engine.url import URL
 
@@ -30,6 +30,13 @@ class MySQLSourceImpl(BaseSQLSource, TableIngestMixIn):
         self.ingested_tables: Dict[str, Any] = {}
         super(MySQLSourceImpl, self).__init__()
 
+    def engine_args(self) -> Mapping[str, Any]:
+        return dict(
+            pool_size=20,
+            max_overflow=60,
+            pool_pre_ping=True,
+        )
+
     def local_engine_url(self) -> Union[str, URL]:
         return URL.create(
             drivername="mysql+pymysql",
