@@ -141,7 +141,7 @@ class Windowed(Generic[T], metaclass=WindowedMeta):
     def __init__(
         self,
         buckets: List[str],
-        mode: Union[Literal["tumbling"], Literal["continuous"]],
+        mode: Literal["tumbling", "continuous"],
         description: Optional[str],
         owner: Optional[str],
         tags: Optional[Any],
@@ -191,7 +191,7 @@ class SelectedWindow:
 
 def windowed(
     *buckets: str,
-    mode: Union[Literal["tumbling"], Literal["continuous"]] = "tumbling",
+    mode: Literal["tumbling", "continuous"] = "tumbling",
     description: Optional[str] = None,
     owner: Optional[str] = None,
     tags: Optional[Any] = None,
