@@ -3,7 +3,9 @@ import os
 import pathlib
 import typing
 import warnings
-from typing import Dict, List, NamedTuple, Optional, Tuple, Type, Union, get_args, get_origin
+from typing import Dict, List, NamedTuple, Optional, Tuple, Type, Union
+
+from typing_extensions import get_args, get_origin
 
 from chalk.features import Feature, Features, FeatureSetBase
 from chalk.importer import get_directory_root, import_all_python_files_from_dir
