@@ -1,9 +1,15 @@
 import polars
+import pytest
 from pydantic import BaseModel
 
 from chalk.features import DataFrame, features
 from chalk.streams import KafkaSource, Windowed, stream, windowed
 
+try:
+    import duckdb
+except ImportError:
+    duckdb = None
+
 
 @features
 class StoreFeatures:
@@ -29,6 +35,7 @@ def fn(messages: DataFrame[KafkaMessage]) -> DataFrame[StoreFeatures.id, StoreFe
     """
 
 
+@pytest.mark.skipif(duckdb is None, reason="duckdb is not installed")
 def test_runs_sql():
     expected = DataFrame(
         polars.DataFrame(
